function find_minima()

global fmin fminxyz  % fsad fsadxyz

Na = 11; % the number of atoms for with you want to find energy minima

% Do the first run with unbiased search, i.e., set 
%
% UNBIASED_SEARCH = 1;
% FLAG_BASIN_HOPPING = 0;
%
% and the second run with basin hopping, i.e. set
%
% UNBIASED_SEARCH = 0;
% FLAG_BASIN_HOPPING = 1;
%
% If you are doing basin hoppin pick the starting minimum (the seed)
seed0 = 1;

fig = 1; % figure on which found minima will be displayed
% If you do not want to see the minima, comment drawLJconf(xyz,fig) out
% (lines 58, 86, 136)

UNBIASED_SEARCH = 0;
FLAG_BASIN_HOPPING = 1;

Nsteps = 10000; % the number of steps for the unbiased search
iter_max_BFGS_TR = 500;
tol_BFGS_TR = 1e-9;

dtol = 1e-1; % if the distance between x and y is less than dtol, 
% x and y represent the same structure
Vtol = 1e-6;
cgtol = 1e-3;

fmin_name = sprintf('LJ%dmin_data.txt',Na);
fmin_name_xyz = sprintf('LJ%dmin_xyz.txt',Na);




if UNBIASED_SEARCH == 1
    fmin = fopen(fmin_name,'w');
    fminxyz = fopen(fmin_name_xyz,'w');
    flag = 0;
    while flag < 1
        xyz = make_random_configuration(Na);
        [xyz,V,freq,flag] = BFGS_trust_region(xyz,tol_BFGS_TR,iter_max_BFGS_TR);
    end
    xyz = cmass2origin(xyz);
    pgo = point_group_order(xyz);
    Vglobalmin = V;

    Nmin = 1;
    minlist(Nmin,:) = [V, freq, pgo];

    printmin(V,freq,pgo,xyz);

    drawLJconf(xyz,fig);
    str = sprintf('Minimum %d\n',Nmin);
    title(str);

    current_min = 1;

    Vglobalmin = min([Vglobalmin,V]);
    T = 10;
    ss = 0.4;
    ka = 0;
    ke = 2;
    stat(1) = 1;
    for k = 2 : Nsteps
        flag = 0;
        V1 = V;
    %    x = remove_rotations_translations(xyz);
        while flag < 1  
           xyz = make_random_configuration(Na);
           [xyz,V1,freq1,flag] = BFGS_trust_region(xyz,tol_BFGS_TR,iter_max_BFGS_TR);
        end
        ind = find(abs(minlist(:,1) - V1) < Vtol );
        if isempty(ind)
            Nmin = Nmin + 1;
            pgo = point_group_order(xyz);
            minlist(Nmin,:) = [V1, freq1, pgo];
            printmin(V1,freq1,pgo,xyz);
            stat(Nmin) = 1;
            current_min = ind;
            drawLJconf(xyz,fig);
        else
            stat(ind) = stat(ind) + 1;
            current_min = ind;
        end
        fprintf('k = %d\t current_min = %d, Nmin = %d\n',k,current_min,Nmin);
    end
    for k = 1 : Nmin
        fprintf('Min %d: %.10f\t found %d times\n',k,minlist(k,1),stat(k));
    end

else
    minlist = load(fmin_name);
    Nmin = size(minlist,1);
    coord = load(fmin_name_xyz);
end
%% basin_hopping
if FLAG_BASIN_HOPPING == 1
    fmin = fopen(fmin_name,'a');
    fminxyz = fopen(fmin_name_xyz,'a');
    Nmin0 = Nmin;
    Nsteps = 1000;
    T = 0.8;
    ss = 0.4;
    ka = 0;
    index_start = 0;

    for seed = seed0 : Nmin0
        fprintf('seed = %d\n',seed);
        index = index_start;
        current_min = seed;
        V = minlist(seed,1);
        xyz = coord(3*(seed - 1) + 1 : 3*seed,:);
        for k = 1 : Nsteps
            flag = 0;
%            V1 = V;
            while flag < 1 % & abs(V1 - V) < Vtol
               u = ss*rand(size(xyz));
               xyz1 = xyz + u; % perturb the configuration
               [xyz1,V1,freq1,flag] = BFGS_trust_region(xyz1,tol_BFGS_TR,iter_max_BFGS_TR);
            end
            if abs(V1 - V) > Vtol
                % check if this minimum coincides with found earlier
                ind = find(abs(minlist(:,1) - V1) < Vtol );
                if isempty(ind)
                    Nmin = Nmin + 1;
                    pgo = point_group_order(xyz1);
                    minlist(Nmin,:) = [V1, freq1, pgo];
                    printmin(V1,freq1,pgo,xyz1);
                    new_min = Nmin;
                    drawLJconf(xyz1,fig);
                else
                    new_min = ind(1);
                end
                index = index + 1;
                neib(index,:) = [current_min,new_min]; 
                fprintf('k = %d\t current_min = %d, new_min = %d\tNmin = %d\n',k,current_min,new_min,Nmin);
                % decide whether to move to the new minimum or not
                if V1 < V
                    xyz = xyz1;
                    V = V1;
                    current_min = new_min;
                    ka = ka + 1;
                else
                    a = exp(-(V1 - V)/T);
                    u = rand;
                    if u < a % possibly, there was an error: a < u
                        xyz = xyz1;
                        V = V1;
                        ka = ka + 1;
                        current_min = new_min;
                    end
                end
            end
        end
    end
end
fprintf('ka = %d\tNmin0 = %d\tNmin = %d\n',ka,Nmin0,Nmin);

fclose(fmin);
fclose(fminxyz);
end

%%
function xyz1 = cmass2origin(xyz)
    % put the center of mass to the origin
    % each column corresponds to an atom
    [m n] = size(xyz);
    cmass = mean(xyz,2); 
    xyz1 = xyz - cmass*ones(1,n); % shift so that the center of mass is at the origin
end

%%
%%
function printmin(v,freq,pgo,xyz)
global fmin fminxyz
fprintf(fmin,'%.10f\t%.10f\t%d\n',v,freq,pgo);
fprintf(fminxyz,'%.10f\t',xyz(1,:)); fprintf(fminxyz,'\n');
fprintf(fminxyz,'%.10f\t',xyz(2,:)); fprintf(fminxyz,'\n');
fprintf(fminxyz,'%.10f\t',xyz(3,:)); fprintf(fminxyz,'\n');
end



