function [xyz,vsad,freq,omega,flag,V1,V2,x1,x2,freq1,freq2,xyz_startdimer] =  find_saddle_minmodedimer(xyz,eigcond1,R,evecdir)  

% Developed by Sebastian Sousa Castellanos, July 2016

[m,Na] = size(xyz);
tol = 1e-13;
iter_max = 1000; 
iter_maxBFGS = 1000;
flag = 0;
iter = 0;
pmax = 1e-1;

evaltol = 1e-3;


nor = 2;
min = xyz; %Minimum from which saddle will be found. 

x = remove_rotations_translations(xyz);
%x_aux = min(:); % make xyz into column vector (Really big column)
% x = [x2 x3 y3 x4 y4 z4 ... xNa yNa ZNa]';
% x = x_aux([4,7,8,10 : Na*3]);  %Get rid of all zeros
N = length(x);
df = LJgrad(x);


%--------------------------------------------------------------------------%
%Finding the saddle (Minmode/Newton)
tic
tol = 1e-9;
nor = 1;
H = LJhess(x);
[evec,eval] = eig(H);
[esort isort] = sort(diag(eval),'ascend');
min_eval = esort(1);
v = evecdir*evec(:,isort(1));%this is the minmode.
%df = LJgrad(x);
mytol = 1e-4;
%nor = norm(df);
minmodeiter = 0;
minmodeiter_max = 10000;
fprintf('START: minmode: min_eval = %d\t V = %d\n',min_eval,LJpot(x));

flag = 1; % if flag == 1 -- we have a chance to succeed

while (min_eval > eigcond1 ) 
    x = x + R*v;
    H = LJhess(x);
%    df = LJgrad(xnew);
    [evec,eval] = eig(H);
    [esort isort] = sort(diag(eval),'ascend');
    min_eval = esort(1);
    vnew = evec(:,isort(1));%this is the minmode.
    if vnew'*v < 0
        v = -vnew;
    else 
        v = vnew;
    end
%    fnew = LJpot(xnew);
%     x = xnew;
%     f = fnew;
    minmodeiter  = minmodeiter +1;
    if mod(minmodeiter,500) == 0
        fprintf('minmode: # iter = %d\t min_eval = %d\t V = %d\n',minmodeiter,min_eval,LJpot(x));
    end
    if minmodeiter > minmodeiter_max
        break;
    end
end

%Shrinking dimer, endpoints oriented along min-mode initially.
x1 = x + R*v;
x2 = x - R*v;
fprintf('minmode: # iter = %d\t min_eval = %d\t V = %d\n',minmodeiter,min_eval,LJpot(x));

x_aux = [0;0;0;x(1);0;0;x(2:3);0;x(4:N)];
xyz_startdimer = reshape(x_aux,3,Na);


[x, vsad,iter] = dimer(x1,x2);
df = LJgrad(x);
nor = norm(df);
%f = LJpot(xnew);

if nor > tol
    %% Newton's method
    tol = 1e-9;
    newtoniter_max = 5;
    newtoniter = 0;
    feval = 0;
    while nor > tol & newtoniter < newtoniter_max
        % find Newton's direction
        H = LJhess(x);
        cH = cond(H);
        if cH > 1e14
            fprintf('cond(Hess) = %d\n',cH);
            flag = 0;
            break;
        end
        p = -H\df;
        nop = norm(p);
        if nop > 1
            p = p/nop;
        end
        x = x + p;
%        fnew = LJpot(xnew);
        feval = feval + 1;
        df = LJgrad(x);
        nor = norm(df);
        newtoniter = newtoniter + 1;
%        fprintf('Newton step size: %d\t |df| = %d, V = %d\n',norm(p),nor,LJpot(x));
    end
end
cputime = toc

if flag == 1
    xsad = x;
    vsad = LJpot(xsad);
    H = LJhess(xsad);
    [evec,eval] = eig(H);
    [evalsort,isort] = sort(diag(eval),'ascend');
    ind1 = find(evalsort < -evaltol);
    neg = length(ind1);
    ind2 = find(evalsort > evaltol);
    pos = length(ind2);
    if neg == 1 & neg + pos == length(x) % Morse index 1 saddle is found
        omega = -evalsort(1);
        freq = sum(log(abs(evalsort)));
        dirneg = evec(:,isort(1));
        x = steepest_descend(xsad + .01*dirneg);
        [x1, V1, freq1, flag1] = BFGS_trust_region(x);
        x = steepest_descend(xsad - .01*dirneg);
        [x2, V2, freq2, flag2] = BFGS_trust_region(x);
        x_aux = [0;0;0;xsad(1);0;0;xsad(2:3);0;xsad(4:N)];
        xyz = reshape(x_aux,3,Na);
        if flag1 == 0 | flag2 == 0
            flag = 0;
        end

    else
        flag = 0;
    end
end

if flag == 0
    xsad = 0;
    vsad = 0;
    freq = 0;
    omega = 0;
    V1 = 0;
    V2 = 0;
    freq1 = 0;
    freq2 = 0;
    x1 = 0;
    x2 = 0;
    neg = 0;
    pos = 0;
end
fprintf('Newton: flag = %d, V = %.10f, neg = %d, pos = %d\n',flag,vsad,neg,pos);
    
end
 
%%

function [xsad, vsad,iter] = dimer(x1,x2)
% fprintf('IN DIMER\n');
    % Shrinking Dimer Method
    [m,n]= size(x1);
    Na = n/3;
    x = 0.5*(x1 + x2);
    l = norm(x1 - x2);
    v = (x2 - x1)/l;

    nor = norm(LJgrad(x));
    tol = 1e-4;
    h = 1e-3;
    iter = 0;
    iter_max = 20000;
    
    %Dimer dynamics...look at notes. (Dimer Paper)
    while nor > tol & iter < iter_max
        F1 = (-LJgrad(x - 0.5*l*v));
        F2 = (-LJgrad(x + 0.5*l*v));
        F = 0.5*(F1 + F2);
        DF = (F2 - F1)/l;
        xnew = x + h*(F - 2*(v*F')*v);
        w = v + h*(DF - (v*DF')*v);
        vnew = w/norm(w);
        lnew = l/(1 + h);
        x = xnew; v = vnew; l = lnew;
        iter = iter + 1;
        nor = norm(LJgrad(x));
%         if mod(iter, 1000) == 0
%              fprintf('iter %d:\t norm(LJgrad(x)) = %d,\t V(x) = %d\n',iter,nor,LJpot(x));
%         end
    end
    xsad = x;
    vsad = LJpot(x);
    fprintf('dimer: # iter = %d\t |grad f| = %d\t V = %d\n',iter, nor,vsad);
end

%%
%%
function [xyz Vxyz freq flag] = BFGS_trust_region(x)
    N = length(x);
    Na = N/3 + 2;
    tol = 1e-9;
    iter_max = 1000;
    [f, g] = LJ(x);
    nor = norm(g);
    Rmax = 0.1; % the maximal trust region radius
    R = 0.05;
    Rmin = 1e-14;
    rho_good = 0.75;
    rho_bad = 0.25;
    eta = 0.01;

%     fprintf('Initially, V = %d\t, |grad V| = %d\n',f,nor); 
    %% The trust region BFGS method
    tic
    
    iter = 0;
    feval = 0;
    flag = 0;
    I = eye(length(x));
    B = I;
    while nor > tol & iter < iter_max
        % solve the constrained minimization problem using dogleg strategy
        pu = -g*(g'*g)/(g'*B*g);
        pb = -B\g;
        normpu = norm(pu);
        normpb = norm(pb);
        if normpu > R 
            p = R*pu/normpu;
        else
            if normpb < R
                p = pb;
            else
                d = pb - pu;
                alp = max(roots([d'*d, 2*pu'*d, pu'*pu - R^2]));                
                p = pu + alp*d;
            end
        end
        iter = iter + 1;        
        % assess the progress
        xnew = x + p;
        [fnew, gnew] = LJ(xnew);
        mnew = f + g'*p + 0.5*p'*B*p;
%        fprintf('f = %d\tfnew = %d\t mnew = %d\n',f,fnew,mnew);
        rho = (f - fnew+1e-14)/(f - mnew+1e-14);
        % adjust the trust region
        if rho < rho_bad
            R = max([0.25*R,Rmin]);
        else
            if rho > rho_good & abs(norm(p) - R) < tol
                R = min([Rmax,2*R]);
            end
        end
        % accept or reject step
        if rho > eta            
            s = xnew - x;
            y = gnew - g;
            aux = B*s;
            B = B - (aux*aux')/(s'*aux) + y*y'/(y'*s);
            x = xnew;
            f = fnew;
            g = gnew;
            nor = norm(g);
%            fprintf('Accept: iter = %d, rho = %d\t f = %d, |grad f| = %d, R = %d\n',iter,rho,f,nor,R);
        else
%            fprintf('Reject: iter = %d,rho = %d\n',iter,rho);
        end
    end
    if iter == iter_max
        flag = -1;
    end
    cputime = toc;
    %% check what is found
    if flag == 0
        fprintf('Result: f = %.8f\t |df| = %d\t niter = %d\n',f,nor,iter);
        H = LJhess(x);
        ihess = sort(eig(H),'ascend');
        freq = sum(log(abs(ihess)));
        if ihess(1) > 0 % if a nondegenerate minimum is found
            flag = 1;
            MorseIndex = 0;
        else
            flag = -1;
        end
        Vxyz = LJpot(x);
        x_aux = [0;0;0;x(1);0;0;x(2:3);0;x(4:N)];
        xyz = reshape(x_aux,3,Na);
    else
        fprintf('BFGS trust region method failed to find a minimum\n');
        xyz = 0;
        freq = 0;
        flag = 0;
        Vxyz = 0;
    end
        

end

%%

function x = steepest_descend(x)
tol = 1e-1;
iter = 0;
iter_min = 50;
g = LJgrad(x);
nor = norm(g);
dt = 1e-3;
% fprintf('SD: iter = %d\t V = %d\n',iter,LJpot1(x));

while iter < iter_min | nor > tol
    x = x - dt*g;
    g = LJgrad(x);
    nor = norm(g);
    iter  = iter + 1;
%    fprintf('SD: iter = %d\t V = %d\n',iter,LJpot1(x));

end
fprintf('SD: iter = %d\t V = %d\n',iter,LJpot(x));
end

%%

%% Lennard-Jones potential        

function v = LJpot(xyz) % 
[m n] = size(xyz);
Na = (m + 6)/3 ;
x_aux = [0;0;0;xyz(1);0;0;xyz(2:3);0;xyz(4:m)];
m = length(x_aux);
% restore atomic coordinates 
x = reshape(x_aux,3,Na);
r = zeros(Na); 
%Matrix of pair wise distances:
for k = 1 : Na
    r(k,:) = sqrt(sum((x - x(:,k)*ones(1,Na)).^2,1)); %the one is direction one, down the column.
end
r = r + diag(ones(Na,1)); 
aux = 1./r.^6;
L = (aux - 1).*aux;
L = L - diag(diag(L)); %Get rid of what is in line 179, turns them back to zeros.
v = 2*sum(sum(L)); %Just by two instead of four, since we counted each distance between atoms twice.
end
 
 %%
function dv = LJgrad(xyz) % 
[m n] = size(xyz);
Na = (m + 6)/3 ;
x_aux = [0;0;0;xyz(1);0;0;xyz(2:3);0;xyz(4:m)];
m = length(x_aux);
% restore atomic coordinates
g = zeros(size(x_aux));
x = reshape(x_aux,3,Na);
r = zeros(Na);
for k = 1 : Na
    r(k,:) = sqrt(sum((x - x(:,k)*ones(1,Na)).^2,1));
end
r = r + diag(ones(Na,1));
L = -6*(2./r.^6 - 1)./r.^8;
for k = 1 : Na
    Lk = L(:,k);
    g(1 + (k-1)*3 : k*3) = (x(:,k)*ones(1,Na) - x)*Lk;
end

g = 4*g;
dv = g([4,7,8,10 : Na*3]);
end

%%
function [v dv] = LJ(xyz) % 
[m n] = size(xyz);
Na = (m + 6)/3 ;
x_aux = [0;0;0;xyz(1);0;0;xyz(2:3);0;xyz(4:m)];
m = length(x_aux);
% restore atomic coordinates 
x = reshape(x_aux,3,Na);
r = zeros(Na);
for k = 1 : Na
    r(k,:) = sqrt(sum((x - x(:,k)*ones(1,Na)).^2,1));
end
r = r + diag(ones(Na,1));
aux = 1./r.^6;
L = (aux - 1).*aux;
L = L - diag(diag(L));
v = 2*sum(sum(L));

g = zeros(size(x_aux));
L = -6*(2./r.^6 - 1)./r.^8;
for k = 1 : Na
    Lk = L(:,k);
    g(1 + (k-1)*3 : k*3) = (x(:,k)*ones(1,Na) - x)*Lk;
end

g = 4*g;
dv = g([4,7,8,10 : Na*3]);

end
 %%
function H = LJhess(x) % find the Hessian using finite differences
h = 1e-4;
n = length(x);
H = zeros(n);
e = eye(n);
for i = 1 : n
    di = e(:,i)*h;
    Hei = 0.5*(LJgrad(x + di) - LJgrad(x - di))/h;
    for j = 1 : i
        H(j,i) = e(j,:)*Hei;
        H(i,j) = H(j,i);
    end
end
end
