function xyz = make_random_configuration(Na)
    rstar = 2^(1/6);
    xyz = zeros(3,Na);
    xyz(:,1) = [0;0;0];
    a = randn(3,Na - 1);
    rad = sqrt(a(1,:).^2 + a(2,:).^2 + a(3,:).^2);
    a = a*diag(1./rad);
    hR = 0.01;
    for i = 2 : Na
        clear rad
        clear x
        rad = sqrt(xyz(1,1 : i - 1).^2 + xyz(2,1 : i - 1).^2 + xyz(3,1 : i - 1).^2);
        R = max(rad) + rstar;
        xa = R*a(:,i - 1);
        x = [xyz(:,1 : i - 1), xa];
        f = LJpot(x);
        fnew = f;
        while 1
            R = R - hR;
            xa = R*a(:,i - 1);
            x = [xyz(:,1 : i - 1), xa];
            f = fnew;
            fnew = LJpot(x);
            if fnew > f
                break;
            end
        end
        xyz(:,i) = xa;
    end
    cmass = mean(xyz,2);
    xyz = xyz - cmass*ones(1,Na);
end

%%
  %% Lennard-Jones potential  
function v = LJpot(xyz) % 
[m n] = size(xyz);
x = xyz(1,:); 
y = xyz(2,:); 
z = xyz(3,:); 
r = zeros(n);

v = 0;
for k = 2 : n
    for j = 1 : k-1
        r(j,k) = sqrt((x(k)-x(j)).^2+(y(k)-y(j)).^2+(z(k)-z(j)).^2);
        r(k,j) = r(j,k);
        v = v + r(j,k).^(-12) - r(j,k).^(-6);
    end
end
v = v*4;
end

%%
function drawconf(xyz)
figure;
hold on;
rstar = 2^(1/6);
r = 0.5*rstar;
x = xyz(1,:); y = xyz(2,:); z = xyz(3,:);
na = length(x);
Mx=max(max(x))+1;
mx=min(min(x))-1;
My=max(max(y))+1;
my=min(min(y))-1;
Mz=max(max(z))+1;
mz=min(min(z))-1;
[x1,x2,x3]=sphere;
x1=x1*r; x2=x2*r;x3=x3*r;
for j=1:na
    surf(x1+x(j),x2+y(j),x3+z(j)); 
end
set(gca,'DataAspectRatio',[1 1 1]);
axis([mx Mx my My mz Mz]);
drawnow
end
